library(SimInf)
library(dplyr)

## ****************************************************************************
## Models, Sampling
## ****************************************************************************

##' SISe model on the 1600 nodes with actual events
##'
##' @param by sampling frequency
SISe1600 <- function(by=1,...){
    ## initialize
    gdata <- c(beta=0.054,gamma=0.1, rho=0.44)
    prev0 <- 0.02

    nodes <- SimInf::nodes ## loads nodes
    events <- SimInf::events_SISe() ## events
    E <- matrix(c(1, 1, 0, 1),
            nrow = 2,
            dimnames = list(c("S", "I")),
            byrow=TRUE)

    u0 <- SimInf::u0_SISe() ## inital states
    phi <- 0 ## low initial prevalence, thus assume no pressure from start.
    v0 <- data.frame(phi=rep(phi,times=dim(nodes)[1]))

    S <- u0$S
    I <- round(S*prev0)
    S <- S-I
    u0$S <- S; u0$I <- I
    tspan <- seq(1,4*365,by=by)

    ## Define the SISe model without phi diffusion between nodes.
    ## This is an alternative to using the predifed SISe_sp defined
    ## in SimInf
    model <- SimInf::mparse(transitions=c("S -> beta*S*phi -> I",
                                          "I -> gamma*I -> S"),
                            compartments=c("S","I"),
                            pts_fun = c("v_new[0] = 1.0*u[1]/(u[0]+u[1]) + v[0]*(1.0 - gdata[2]);",
                                        "return v_new[0] != v[0];"),
                            gdata=gdata,
                            u0=u0,
                            v0=v0,
                            tspan=tspan,
                            events = events,
                            E = E)

    priors <- c(beta ~ uniform(0, 1),
                gamma ~ uniform(0, 1),
                rho ~ uniform(0.4, 0.5))

    return (list("model"=model,"priors"=priors,"sigma"=1))
}

##' SIS model on the 1600 nodes with actual events
##'
##' @param by sampling frequency
SIS1600 <- function(by=1,...){
    ## initialize
    gdata <- c(beta=0.16,gamma=0.1)
    prev0 <- 0.1

    nodes <- SimInf::nodes ## loads nodes

    events <- SimInf::events_SISe() ## events
    E <- matrix(c(1, 1, 0, 1),
                nrow = 2,
                dimnames = list(c("S", "I")),
                byrow=TRUE)

    u0 <- SimInf::u0_SISe() ## inital states

    S <- u0$S
    I <- round(S*prev0)
    S <- S-I
    u0$S <- S; u0$I <- I
    tspan <- seq(1,4*365,by=by)

    ## rparse define the model, and allows for network effects.
    model <- SimInf::mparse(transitions=c("S -> beta*S*I/(S+I) -> I",
                                          "I -> gamma*I -> S"),
                            compartments=c("S","I"),
                            gdata = gdata,
                            u0 = u0,
                            tspan = tspan,
                            events = events,
                            E = E)

    priors <- c(beta ~ uniform(0, 1),
                gamma ~ uniform(0, 1))
    return (list("model"=model,"priors"=priors,"sigma"=1))
}

##' Ornstein-Uhlenbeck LS estimates
##'
##' @param X time-series data
##' @param tt time vector
##' @param dt time between element in tt, can be left empty if tt is
##'     suplied.
##' @return minsLS: 3 parameter estimate
##' @export
OU_LS <- function(X, tt=NULL, dt=NULL) {
    if(is.null(dim(X)))
        X = as.matrix(X)
    dx <- t(diff(X))


    if(is.null(tt) && is.null(dt))
        stop()
    if (is.null(dt))
        dt <- diff(tt)


    h <- mean(dt) ## should be constant
    dy <- dx/dt
    N <- dim(dx)[2]
    M <- dim(dx)[1]


    A <- matrix(c(matrix(1,N,M), -X[1:N,]),N)

    kmu_ <- pracma::pinv(t(A)%*%A)%*%t(A)%*%t(dy)


    sigma_ <- sqrt(1/(N-2)*sum(dt*(A%*%kmu_-t(dy))^2));
    minsLS_ <- matrix(c(kmu_, sigma_),1)
    minsLS <- array(0,dim(minsLS_))
    minsLS[2] <- (1-exp(-minsLS_[2]*h))/h; ## mu
    minsLS[1] <- minsLS[2]*(minsLS_[1]/minsLS_[2]); ## k
    minsLS[3] <- minsLS_[3]*sqrt(minsLS[2]*(2-minsLS[2]*h)/(2*minsLS_[2])); ## sigma


    return(minsLS)
}


##' summary statistics function assuming and potential binary
##' measurement on the data
##' @param x the observation
##' @param weights the weights per node
##' @param cutoff the cutoff in binary filter
##' @param skip burn-in period
##' @param tspan_sample what time points to consider
##' @param by dt length
sumstat <- function(x,weights,cutoff,skip,tspan_sample,by){
    ## define the sampling scheme
    ## cut = 0.45*sigma
    ##cut = 0.3*sigma
    measurement <- function(x,c=cutoff){
        if (c == 0) # no filter
            return(x)
        return(as.numeric(x>c)*c)
    }

    x$weights <- weights
    ## apply measurement
    x.m <- x %>%
        mutate(y=weights*measurement(prevalence),
               y_=weights*prevalence) %>%
        group_by(time) %>%
        filter(time >= skip) %>%
        filter(time %in% tspan_sample) %>%
        summarize(y=sum(y),y_=sum(y_))


    x.ss <- x.m %>% select(y) %>% as.matrix() %>% OU_LS(dt=by)

    return(list("m"=x.m,"ss"=x.ss))
}

##' Run the ABC-SMC code from the SimInf package
runabc <-  function(ngen=5,npart=1000,tol=10,ptol=0.9,verbose=FALSE,
                    modelfn=SISe,cutoff=0.3,skip=0,nObs=NULL,
                    by=1) {
    model <- modelfn(by=1) # run every day
    priors <- model$priors
    sigma <- model$sigma ## update sigma from modelfn
    model <- model$model ## after extraction, write over for name clarity

    ## sample every "by" days. CV suggest by < 6
    ## see beta in covtest(5)$cv, covtest(6)$cv
    tspan_sample <- model@tspan[seq(1,max(model@tspan),by=by)]



    ## Run the SimInf model and generate the observation
    set.seed(0)
    if (is.null(nObs)) {
        index <- seq_len(dim(model@u0)[2])
    } else if (nObs > dim(model@u0)[2]) {
        stop("nObs is larger than network size")
    } else {
        index <- sample(seq_len(dim(model@u0)[2]),nObs)
    }
    index <- sort(index)

    ## compute the weight given per measurement
    all <- trajectory(run(model),index=index)
    N <- all %>% mutate(N=S+I) %>% select(N,node,time)
    N_pertime <- N %>% group_by(time) %>% summarize(N=sum(N)) %>% select(N) %>% as.matrix()
    weight_mat <- matrix(as.numeric(N$N),nrow=length(index))

    weights <- weight_mat %>% sweep(2,N_pertime,"/")
    weights <- c(weights)

    ## generate observation
    set.seed(22)
    ##obs <- trajectory(run(model), compartments=c("I"), index=index)
    obs <- prevalence(run(model), I~.,level=3, index=index)







    ## distance between two samples, normalized euclidian
    euclidn <- function(x,y){
        dist <- sqrt(sum( ((x-y)/x)^2 ))
        if (is.nan(dist))
            dist <- Inf
        return(dist)
    }

    ## summary statistics of the observation used in the ABC schema
    sumstat_run <- function(x){
        sumstat(x,weights=weights,cutoff=cutoff,
                skip=skip,tspan_sample=tspan_sample,by=by)
        }

    obs.ss <- sumstat_run(obs)$ss


    ## after simulation, determine the SS and return accept or reject
    ## of the proposal.
    acceptFun <- function(result, generation, tol, ptol, extra,...) {
        ## Determine the tolerance for this generation.
        if ("tol" %in% names(extra))
            tol <- extra$tol[generation] ## full vector defined
        else
            tol <- tol * (ptol)^(generation - 1) ## only "function" defined

        ## get the prevalence and apply filter
        sim <- prevalence(result, I~.,level=3, index=index)
        sim.ss <- sumstat_run(sim)$ss
        ## normalized euclidian distance
        dist <- euclidn(obs.ss,sim.ss)


        ## Return TRUE or FALSE statement to SimInf
        SimInf::abc_accept(dist < tol, tol)
    }

    ## Find SMC-ABC fit
    fit <- SimInf::abc(model = model,
                       priors = priors,
                       ngen = ngen,
                       npart = npart,
                       fn = acceptFun,
                       tol = tol,
                       ptol = 0.9, ## not in use
                       verbose = verbose,
                       extra=list("tol"=tol))

    return(list("fit"=fit,"obs"=obs,"gdata"=model@gdata,
                "index"=index,"tspan_sample"=tspan_sample))
}

##' Generate exponentially decaying tolerance vector
##' @param t0 starting point
##' @param a decay factor
##' @param N number of points generated
exptol <- function(t0=100,a=0.5,N=20) {
    return(t0*exp(-a*seq(0,N)))
}

## ****************************************************************************
## Plotting
## ****************************************************************************
library(ggplot2)
library(scales)
library(reshape2)
library(ggridges)
library(dplyr)

loadfont <- function() {
    library(extrafont)
    library(remotes)
    extrafont::font_import(pattern="lmroman*")
    loadfonts(device="pdf")
}


##' theme:ing for ggplots
##' Initial Credit goes to:
##' Koundinya Desiraju (04/07/2015)
##' Later adjustments have been made by Robin Eriksson.
##' @export
theme_Publication <- function(base_size=14) {
    (ggthemes::theme_foundation(base_size=base_size)
        + ggplot2::theme(plot.title = ggplot2::element_text(face = "bold",
                                                            size = ggplot2::rel(1.2),
                                                            hjust = 0.5),
                         text = ggplot2::element_text(family="LM Roman 10"),
                         panel.background = ggplot2::element_rect(colour = NA),
                         plot.background = ggplot2::element_rect(colour = NA),
                         panel.border = ggplot2::element_rect(colour = "black"),
                         axis.title = ggplot2::element_text(face = "bold",size = ggplot2::rel(1)),
                         axis.title.y = ggplot2::element_text(angle= 90,vjust =0),
                                        #axis.title.x = ggplot2::element_text(vjust = 4),
                         axis.text = ggplot2::element_text(),
                         axis.text.x = ggplot2::element_text(angle = 0),
                         axis.line = ggplot2::element_line(colour=NA),
                         axis.ticks = ggplot2::element_line(colour = "black", size = 0.2),
                         axis.ticks.length=unit(-0.075, "cm"),
                         panel.grid.major = ggplot2::element_line(colour=NA),
                         panel.grid.minor = ggplot2::element_blank(),
                         legend.background = ggplot2::element_rect(fill='transparent'),
                         legend.box.background = ggplot2::element_rect(colour="black",
                                                                       fill='white',
                                                                       size=0.3),
                         legend.key = ggplot2::element_rect(colour = "transparent"),
                         ##legend.box.background = ggplot2::element_rect(colour = NA,fill = alpha("red", 0)),
                         legend.position = c(0.2945,0.9075),#c(0.1045,0.775),#"bottom",
                         legend.direction = "horizontal",
                         ##legend.key.size= grid::unit(0.5, "cm"),
                         legend.key.width=grid::unit(0.2,"cm"),
                         legend.key.height=grid::unit(0.2,"cm"),
                                        #legend.title = ggplot2::element_text(face="italic"),
                         plot.margin=grid::unit(c(1,1,1,1),"mm"),#grid::unit(c(10,5,5,5),"mm"),
                         strip.background= ggplot2::element_rect(colour="#f0f0f0",fill="#f0f0f0"),
                         strip.text = ggplot2::element_text(face="bold")
                         ))

}

##' Generate network illustration. (Figure 5.3)
##' @param save the figure to file
network <- function(save=FALSE){
    fit_sise1600 <- readRDS("fit_sise1600.rds")

    all <- SimInf::nodes ## loads nodes

    ## selected nodes
    selected <- fit_sise1600$index

    nodes <- data.frame(all)
    nodes$type <- "not obs"
    nodes$type[selected] <- "obs"
    nodes$alpha <- "A"
    nodes$alpha[selected] <- "D"
    ## events (transports)
    events <- SimInf::events_SISe() ## events
    ## select only a few transports
    events_plot <- events %>% filter(event == "extTrans",time==1) %>%
        select(node,dest) %>% head(15)

    ## prepare
    events_plot$x1 <- nodes$x[events_plot$node]
    events_plot$x2 <- nodes$x[events_plot$dest]
    events_plot$y1 <- nodes$y[events_plot$node]
    events_plot$y2 <- nodes$y[events_plot$dest]


    ## for color coding
    events_plot$type <- "not obs"
    events_plot$type[c(events_plot$node %in% selected %>% which,
                       events_plot$dest %in% selected %>% which)] <- "obs"
    events_plot$alpha <- "B"



    ## generate plot
    p <- ggplot(nodes,aes(x,y,color=type,alpha=alpha)) +
        geom_point() +
        geom_curve(mapping=aes(x = x1, y = y1, xend = x2, yend = y2),
                   data = events_plot,#color="blue",
                   linetype="solid") +
        scale_color_manual(values=c("#000000", "#F54236")) +
        theme_Publication(12) +
        theme(axis.title.y = element_blank(),
              axis.text.y = element_blank(),
              axis.ticks.y = element_blank(),
              axis.title.x = element_blank(),
              axis.text.x = element_blank(),
              axis.ticks.x = element_blank(),
              legend.position="none")



    if(save)
        ggsave(plot=p,filename="network_illustration.pdf",
               family="LM Roman 10", width=2.20,height=2.20)

    return(p)

}


##' Generate measurement illustration. (Figure 5.4)
plotPrevalence <-  function(save=FALSE) {
    seed <- 4
    model <- SISe1600(by=1)$model # run every day
    cutoff <- 0.045
    skip <- 0
    nObs <- 100
    by <- 7
    Nsamples <- 5


    print("Preparing data ...")
    ## sample every "by" days. CV suggest by < 6
    ## see beta in covtest(5)$cv, covtest(6)$cv
    tspan_sample <- model@tspan[seq(1,max(model@tspan),by=by)]



    ## Run the SimInf model and generate the observation
    set.seed(0)
    if (is.null(nObs)) {
        index <- seq_len(dim(model@u0)[2])
    } else if (nObs > dim(model@u0)[2]) {
        stop("nObs is larger than network size")
    } else {
        index <- sample(seq_len(dim(model@u0)[2]),nObs)
    }
    index <- sort(index)

    print(" ... Generating SimInf samples")
    ## compute the weight given per measurement
    all <- trajectory(run(model),index=index)
    N <- all %>% mutate(N=S+I) %>% select(N,node,time)
    N_pertime <- N %>% group_by(time) %>% summarize(N=sum(N)) %>% select(N) %>% as.matrix()
    weight_mat <- matrix(as.numeric(N$N),nrow=length(index))

    weights <- weight_mat %>% sweep(2,N_pertime,"/")
    weights <- c(weights)



    ## generate observation
    set.seed(seed)
    ##obs <- trajectory(run(model), compartments=c("I"), index=index)
    obs <- prevalence(run(model), I~.,level=3, index=index)

    ## summary statistics of the observation used in the ABC schema
    obs.ss <- sumstat(obs,weights=weights,cutoff=cutoff,
                      skip=skip,tspan_sample=tspan_sample,by=by)
    df <- obs.ss$m
    colnames(df) <- c("time","Filtered","Unfiltered")


    ##' Generate a OU realisation
    ##' @param k mean reversion point parameter
    ##' @param mu scaling parameter
    ##' @param sigma noise parameter
    ##' @param tspan time vector
    ##' @param X0 starting value
    genOU <- function(k,mu,sigma,tspan,X0=1) {
        ## (1) compute corrected numerical parameters per each time step such
        ## that a forward Euler step yields an exact step of the wanted
        ## OU-process
        dt = diff(tspan)
        mu_ = (1-exp(-mu*dt))/dt
        k_ = mu_*(k/mu)
        sigma_ = sigma*sqrt(mu_*(2-mu_*dt)/(2*mu))
        ## (solved from Prop. 2.1 in [1])

        ##  (2) then do forward Euler using these numerical parameters
        xx = X0
        x = c(X0, length(dt))
        for (i in seq_len(length(dt))) {
            xx = xx+(k_[i]-mu_[i]*xx)*dt[i]+sigma_[i]*sqrt(dt[i])*rnorm(1);
            x[i+1] = xx;
        }
        return(data.frame(time=tspan,x=x))
    }
    print("... Generating OU samples")
    ou <- replicate(Nsamples,
                    genOU(k=obs.ss$ss[1],mu=obs.ss$ss[2],sigma=obs.ss$ss[3],
                          tspan=tspan_sample,X0=0),
                    FALSE)
    df_ou <- data.frame(ou[[1]])
    if(Nsamples > 1) {
        for (i in seq(2,Nsamples))
            df_ou <- df_ou %>% left_join(ou[[i]],by="time")
        colnames(df_ou) <- c("time",paste("x",1:Nsamples,sep="."))
    }

    ## get scaling for multiple y-axes
    scale <- 3.5


    df$Filtered <- df$Filtered * scale
    df_ou[,-1] <- df_ou[,-1] * scale



    ## melt to enable multiple series plotting
    df.melt <- melt(df,id.vars="time")
    df_ou.melt <- melt(df_ou,id.vars="time")

    colnames(df.melt)[c(1,3)] <- c("t","Value")
    colnames(df_ou.melt)[c(1,3)] <- c("t", "Value")

    print("Constructing plot ...")
    breaks <- c(0.04,0.05,0.06,0.07)
    limits <- c(0.038,0.072)
    p <- ggplot(df.melt,aes(x=t,y=Value,color=variable)) +
        geom_line(mapping=aes(x=t,y=Value,color=variable),
                  alpha=0.2, data=df_ou.melt) +
        geom_line() +
        scale_color_manual(values=c("blue","red",rep("black",Nsamples))) +
        scale_y_continuous(limits=limits,expand=c(0,0.01),
                           guide=guide_axis(check.overlap = TRUE)) +
        scale_x_continuous(expand=c(0,0))

    ## add secondary axis
    breaks2 <- c(1.1,1.4,1.7,2.0)/100
    p <- p +  scale_y_continuous(limits=limits,
                                 expand=c(0,0),
                                 guide=guide_axis(check.overlap = TRUE),
                                 breaks=breaks,
                                 labels=function(x) paste0(100*x,"%"),
                                 sec.axis = sec_axis(~./scale,
                                                     name="Pseudo prevalence",
                                                     breaks=breaks2,
                                                     labels=function(x) paste0(round(100*x,2),"%"),
                                                     guide=guide_axis(check.overlap = TRUE))) +
        labs(y="Prevalence", color="") +
        theme_Publication(12) +
        theme(legend.position="none",
              axis.text.x = element_blank(),
              axis.ticks.x = element_blank(),
              axis.title.y = element_text(color = "red"),
              axis.title.y.right = element_text(color = "blue"),
              axis.text.y = element_text(color = "red"),
              axis.text.y.right = element_text(color = "blue")
              )

    print("Done")
    if(save)
        ggsave(plot=p,filename="filtered_data.pdf",
               family="LM Roman 10", width=4.40,height=2.2)

    return(p)
}


##' Given a fit object, extract the posterior and add a R0 column
##' @param fit SMC-ABC fit object
##' @param gen what posterior generation to use
R0 <- function(fit,gen) {
    if (is.null(gen))
        gen <- length(fit$posterior)
    post <- fit$posterior[[gen]] %>% t() %>% data.frame
    gdata <- fit$gdata

    if (dim(post)[2] == 3) {
        post$R0 <- sqrt(post$beta/(post$gamma*post$rho))
        gdata_R0 <- sqrt(gdata["beta"]/gdata["gamma"]/gdata["rho"])
    } else {
        post$R0 <- post$beta/post$gamma
        gdata_R0 <- gdata["beta"]/gdata["gamma"]
    }
    return(post)
}

##' Generate R0 posterior illustration. (Figure 5.5)
##' @param gen the posterior generation
##' @param save the figure to file
R0post <- function(gen=15, save=FALSE) {
    ## load SISe fit
    fit_sise1600 <- readRDS("fit_sise1600.rds") # loads: fit_sise1600_bin_100_7th_wide1000


    ## load SIS fit
    fit_sis1600 <- readRDS("fit_sis1600.rds") # loads: fit_sis1600_bin_100_7th




    samplePrior <- function(fit,N=100) {
        priorsHyp <- fit$prior[,c("p1","p2")]
        ncol <- dim(priorsHyp)[1]
        prior <- c()
        for (i in seq_len(ncol))
            prior <- c(prior,runif(N,priorsHyp[i,1],priorsHyp[i,2]))
        prior <- matrix(prior,ncol=ncol) %>% as.data.frame
        names(prior) <- fit$prior[,"parameter"]

        if(ncol > 2) {## SISe
            prior$R0 <- sqrt(prior$beta/prior$gamma/prior$rho)
        } else {## SIS
            prior$R0 <- prior$beta/prior$gamma
        }
        return(prior)
    }

    format <- function(fit,gen) {
        dfpost <- R0(fit,gen)
        dfprior <- samplePrior(fit,N=1e4)
        dfprior <- dfprior %>% filter(R0>0.25 & R0 < 4)

        df <- rbind(dfpost,dfprior)
        N <- dim(dfpost)[1]
        N0 <- dim(dfprior)[1]
        df$type <- c(rep("Posterior",N),rep("Prior",N0))

        gdata <- getgdata(fit)
        df <- rbind(df,gdata)

        if (dim(df)[2] > 4)
            df$model <- "SISe"
        else
            df$model <- "SIS"

        return(df)
    }

    getgdata <- function(fit) {
        gdata <- fit$gdata
        if(length(gdata) > 2) {## SISe
            gdata["R0"] <- sqrt(gdata["beta"]/gdata["gamma"]/gdata["rho"])
        } else {## SIS
            gdata["R0"] <- gdata["beta"]/gdata["gamma"]
        }
        gdata <- data.frame(t(gdata))
        gdata$type <- "Actual"
        return(gdata)
    }

    df <- format(fit_sise1600,gen)


    df2 <- format(fit_sis1600,gen)
    df2$rho <- NA
    df <- rbind(df,df2)



    r0_actual <- df %>% filter(type=="Actual") %>%
        select(R0) %>% round(3)

    g <- ggplot(df,aes(x=R0,fill=type,y=model)) +
        geom_density_ridges(scale=4,alpha=0.5, bandwidth=0.05) +
        geom_vline(aes(xintercept = R0),data = df %>% filter(type=="Actual"),
                   linetype="dashed")+
        geom_vline(aes(xintercept=2.2)) +
        geom_vline(aes(xintercept=0.5)) +
        annotate("text",label="Cutoff",x=2.15,y=3.5,angle=90) +
        annotate("text",label="Cutoff",x=0.54,y=3.5,angle=270) +
        scale_x_continuous(breaks=c(0.5,sort(r0_actual$R0),2.2),
                           limits=c(0.5,2.2),
                           expand=c(0.05,0)) +
        labs(y="",x=expression(R[0]), fill="") +
        scale_fill_manual(values=c("red","blue"))+
        theme_Publication(12) +
        theme(legend.position = c(0.7,0.9115),
              legend.margin = margin(l=1,r=5,t=1,b=1))

    if (save)
        ggsave(plot=g,filename="posterior1600_R0.pdf",
               family="LM Roman 10", width=4.23,height=1.5)
    return(g)
}




##' generate the concentration table. (Figure 5.5)
##' @param gen the posterior generation
reportTable <- function(gen=15) {

    ## generate table entries
    evalpost <- function(fit,gen) {
        post <- R0(fit,gen=gen)
        gdata <- fit$gdata

        if(length(gdata) > 2) {## SISe
            gdata["R0"] <- sqrt(gdata["beta"]/gdata["gamma"]/gdata["rho"])
        } else {## SIS
            gdata["R0"] <- gdata["beta"]/gdata["gamma"]
        }

        ## gdata <- fit$gdata
        med <- apply(post,2,median)
        mu <- apply(post,2,mean)
        sigma <- apply(post,2,sd)


        normbias <- ((mu-gdata)/gdata)^2


        cv <- sigma/mu
        iqr <- apply(post,2,IQR)
        mad <- apply(post,2,mad)

        quant <- apply(post,2,function(x){quantile(x,c(0.25,0.75))})
        qcd <- (quant[2,]-quant[1,]) / (quant[2,]+quant[1,])

        priors <- fit$prior
        priorw <- priors$p2 - priors$p1
        priorw <- c(priorw,NA)
        priorw <- priorw/2 # Uniform dist -> priorw0 is full width and


        df <- data.frame(numeric(5)) ## dummy column
        for (row in seq_len(dim(priors)[1])) {
            if (priors[row,"distribution"] == "uniform") {
                p_quant <- seq(priors[row,"p1"],priors[row,"p2"],length.out=5)
            } else {
                stop("not defined distribution in prior")
            }
            df <- cbind(df,p_quant)
        }

        priors_quant <- df[-c(1,3,5),-1] # drop first dummy column, and 0% and 100% rows
        colnames(priors_quant) <- priors$parameter
        rownames(priors_quant) <- c("25%","75%")


        if ("rho" %in% colnames(priors_quant)) ## SISe model
            priors_quant$R0 = c(3.8,7.6) ## hard coded R0
        else
            priors_quant$R0 = c(0.5,2.0) ## hard coded R0




        priors_qcd <- (priors_quant[2,]-priors_quant[1,]) / (priors_quant[2,]+priors_quant[1,])
        priors_iqr <- priors_quant[2,]-priors_quant[1,]




        df <- rbind(gdata,med,mu,sigma,cv,iqr,mad,qcd,iqr/med,iqr/priorw, qcd/priorw,
                    priors_qcd, qcd/priors_qcd, iqr/priors_iqr, normbias)
        rownames(df) <- c("true","median","mean","sd","CV","IQR","MAD","QCD","IQR/median",
                          "IQR/prior width","QCD/prior width","QCD prior",
                          "post/prior (QCD)", "post/prior (IQR)", "(Normalized bias)^2")


        df
    }

    ## load SIS fit
    fit_sis1600 <- readRDS("fit_sis1600.rds") # loads: fit_sis1600_bin_100_7th
    df_sis <- evalpost(fit_sis1600,gen)
    ## load SISe fit
    fit_sise1600 <- readRDS("fit_sise1600.rds") # loads: fit_sise1600_bin_100_7th
    df_sise <- evalpost(fit_sise1600,gen)

    ## format table
    row <- "post/prior (QCD)"
    df_sis$rho <- NA
    df_sis <- df_sis[,c("beta","gamma","rho","R0")]
    df <- rbind(df_sis[row,],df_sise[row,])
    rownames(df) <- c("SIS","SISe")
    return(df)
}

##' Reduce the size of the objects
##'
##' SimInf objects can be large, and to save space
##' in stored files, we can remove the redundant fields.
##' @param fit runabc fit object
##' @param file the filename of the stored file
fit_essentials <- function(fit,file){
    ## Keep only what is needed to save the results.

    ## 1) the posterior
    fit$posterior <- fit$fit@x

    ## 2) the priors
    fit$prior <- fit$fit@priors

    ## remove the large SimInf object
    fit <- fit[-(which(names(fit) == "fit"))]

    saveRDS(fit,file=file)
    return(0)
}

##' Generate the posterior data
main_fit <- function(){
    fit_sis1600  <- runabc(modelfn=SIS1600,
                           cutoff=0.3,
                           tol=exptol(),
                           npart=1e3,
                           verbose=TRUE,
                           ngen=15,
                           nObs=100,
                           skip=0,
                           by=7)
    fit_essentials(fit_sis1600,file="fit_sis1600.rds")

    fit_sise1600_bin_100_7th <- runabc(modelfn=SISe1600,
                                       cutoff=0.04,
                                       tol=exptol(),
                                       npart=1e3,
                                       verbose=TRUE,
                                       ngen=15,
                                       nObs=100,
                                       skip=0,
                                       by=7)
    fit_essentials(fit_sise1600,file="fit_sis1600.rds")
}

## Generate the paper figures
main_plot <- function() {
    ##' remove if not LM Roman is installed
    loadfont()
    ## Fig 5.3
    network(TRUE)
    ## Fig 5.4
    plotPrevalence(TRUE)
    ## Fig 5.5 (top)
    R0post(gen=15,TRUE)
    ## Fig 5.5 (bottom)
    print(reportTable(gen=15))
}
