%FILTERED_BAYESIAN_KALMAN_SIS Bayesian inference filtered data, SIS-model.

% S. Engblom 2021-08-12 (derived from fullstate_bayesian_kalman_sis.m)
% S. Engblom 2021-07-30

%% 1) generate data

if ~exist('illustrate','var')
  illustrate = true;
end

% setup SIS-model (notation from the main paper)
Sigma = 1000;
if ~exist('R0','var')
  R0 = 1.5;
end
% days^-1 to recovery
gamma = 1/10;
beta = R0*gamma;

% filter cut-off value
c = 0.9*Sigma*(1-1/R0);

Tend = 1000; % note: should be considered relative to gamma
N = Tend+1;
tsample = linspace(0,Tend,N);
h = tsample(2)-tsample(1);
X0 = [NaN; ceil(1e-2*Sigma)]; % [S I]
%X0 = [NaN; ceil(Sigma*(1-1/R0))]; % (start at stationary mean)
X0(1) = Sigma-X0(2);

clear umod
if ~exist('umod','var')
  umod.vol = Sigma;
  umod.D = sparse(zeros(2));
  umod = rparse(umod,{'S+I > Beta*S*I/vol > I+I', ...
                      'I > Gamma*I > S' ...
                      'S > (I == 0) > I'}, ... % (prevents extinction)
                {'S' 'I'},{'Beta' 'gdata' 'Gamma' gamma},'SIS.c');

  umod.sd = 1;
  umod.u0 = X0;
  umod.tspan = tsample;
end
umod.gdata = beta;

% simulate
if ~exist('compile','var')
  compile = true;
end
umod = urdme(umod,'solver','ssa','seed',1,'compile',compile);
Xctmc = umod.U;
Y = Xctmc(2,:) >= c;

% $$$ if illustrate
% $$$   figure(1), clf,
% $$$   plot(tsample,Xctmc(2,:),'r'); hold on
% $$$   yline(c,'k--');
% $$$ end

%% 2) estimate likelihood

% log-likelihood
logL = @SIS_kalman_filtered;

% parameter transformations
% $$$ mu = gamma*(R0-1);
% $$$ k = gamma*Sigma*(R0-1)^2/R0;
% $$$ sigma = sqrt(2*gamma*Sigma*(R0-1)/R0);
% $$$ % $$$ atrue = k/mu;
% $$$ % $$$ btrue = exp(-mu*h);
% $$$ % $$$ gtrue = 2*mu/(sigma^2*(1-exp(-2*mu*h)));
% $$$ if illustrate
% $$$   X_OU = OU_process(k,mu,sigma,tsample,X0(2));
% $$$   plot(tsample,X_OU,'b--');
% $$$ end

% SIS --> OU map
atrue = Sigma*(1-1/R0);
btrue = exp(-gamma*h*(R0-1));
gtrue = (Sigma\R0)/(1-btrue^2);

% compute the log-likelihood over a mesh
Rdom = linspace(1.3,1.7,23);
Sdom = linspace(Sigma*0.9,Sigma*1.1,21);
gdom = gamma;
% finer resolution (comment for speed):
Rdom = linspace(1.3,1.7,71);
Sdom = linspace(Sigma*0.9,Sigma*1.1,61);

% curve const = gamma*(c-alpha)^2
const = gtrue*(c-atrue)^2;

% a fairly similar resolution in OU-parameters
aDom = linspace(atrue*0.975,atrue*1.025,81);
bDom = linspace(btrue*0.9975,btrue*1.0025,81);
gDom = linspace(gtrue*0.95,gtrue*1.05,81);
% $$$ gcurve_ = const./(c-aDom).^2;
% $$$ T = aDom+1./(gcurve_'*(1-bDom.^2)); % Sigma
% $$$ T = 1+reshape(aDom'*gcurve_,[],1)*((1-bDom.^2)); % R0
% $$$ [min(T(:)) max(T(:))]

L = zeros(numel(Rdom),numel(Sdom),numel(gdom),numel(tsample));
hw = waitbar(0,'Computing...');
for i = 1:numel(Rdom)
  waitbar(i/numel(Rdom),hw);
  for j = 1:numel(Sdom)
    for k = 1:numel(gdom)
      g_ = gdom(k);
      b_ = Rdom(i)*g_;
      [L_,xx,W] = logL([b_ g_ Sdom(j) h],Y,c, ...
                       X0(2),1e-6,4,100);
      L(i,j,k,:) = reshape(L_,1,1,1,[]);
    end
  end
end
close(hw);

% $$$ % take a look at the filter estimate
% $$$ if illustrate
% $$$   g_ = gamma;
% $$$   b_ = R0*g_;
% $$$   [L_,xx,W] = logL([b_ g_ Sigma h],Y,c, ...
% $$$                    X0(2),1e-6,50);
% $$$   x_ = xx*W;
% $$$   plot(tsample,x_,'k.--');
% $$$ end

L = cumsum(L,4);
Pr = L(:,:,end);
if any(isnan(Pr(:)))
  warning('There seems to be NaN''s around...');
end

%% 3) postprocessing

% numerical posterior
Pr = exp(Pr-max(Pr(:)));
hR = Rdom(2)-Rdom(1);
hS = Sdom(2)-Sdom(1);
Pr = Pr/sum(Pr(:))/(hR*hS);

% visualize the likelihood/posterior
if illustrate
  figure(2), clf,
  contour(Sdom,Rdom,log(Pr),20); hold on,
  v = axis;

  % curve {beta = btrue,gamma*(c-alpha)^2 = const} in (Sigma,R0,gamma)
  gcurve_ = const./(c-aDom).^2; % so (aDom,btrue,gcurve_)
  S_ = aDom+1./((1-btrue^2)*gcurve_);
  R_ = 1+aDom.*(1-btrue^2).*gcurve_;
  %G_ = log(btrue)/h./(1-R_); % ignored
  plot(S_,R_,'k');
  plot(Sigma,R0,'ko');

% $$$   % investigate what another c_ could contribute
% $$$   for rr = [0.7 1.3]
% $$$     c_ = rr*c; % so new c_ = r*c
% $$$     const_ = gtrue*(c_-atrue)^2;
% $$$     gcurve_ = const_./(c_-aDom).^2;
% $$$     S_ = aDom+1./((1-btrue^2)*gcurve_);
% $$$     R_ = 1+aDom.*(1-btrue^2).*gcurve_;
% $$$     plot(S_,R_,'k--');
% $$$   end
  
  xlabel('$\Sigma$','interpreter','latex');
  ylabel('$R_0$','interpreter','latex');
  axis(v);
  hca = gca;
  set(hca,'xtick',900:100:1100);
  set(hca,'ytick',1.3:0.1:1.7);
  set(gcf,'PaperPositionMode','auto');
  set(gcf,'Position',[100 100 350 170]);
end

% marginal density R0
Pr1_ = sum(Pr,2)*hS; % "reasonable" scaling 

% immediate numerical attempt
mom1 = [trapz(Rdom,Pr1_') trapz(Rdom,Pr1_'.*Rdom) trapz(Rdom,Pr1_'.*Rdom.^2)];
mom1(3) = sqrt(mom1(3)-mom1(2)^2);

% normal fit and deduced moments
mom1_ = [mom1; normfit_nomass(Rdom,Pr1_',mom1)];

if illustrate
% $$$   figure(3), clf,
% $$$   Rdom_ = linspace(Rdom(1),Rdom(end),60);
% $$$   plot(Rdom,Pr1_*mom1_(2,1)/mom1(1),'b', ...
% $$$        Rdom_,normpdf(Rdom_,mom1_(2,2),mom1_(2,3)),'r--');
% $$$   xline(mom1_(2,2),'r--');
% $$$   xlabel('$R_0$','interpreter','latex');
% $$$   axis tight; v = axis;
% $$$   v(1:3) = [Rdom(1) Rdom(end) 0]; axis(v);
% $$$   hca = gca;
% $$$   set(hca,'xtick',1.3:0.1:1.7,'xticklabel',{1.3 '' 1.5 '' 1.7});
% $$$   set(hca,'ytick',[]);
% $$$   set(gcf,'PaperPositionMode','auto');
% $$$   set(gcf,'Position',[450 100 110 200]);
 
  % marginal plotted to the left of contour
  f = figure(2);
  % new axes in figure f
  hax2 = axes('parent',f,'units','normalized');

  % scale previous axes and scale the new ones
  pos = get(hca,'position');
  set(hca,'position',[pos(1) pos(2) pos(3)*0.75 pos(4)]);
  pos = get(hca,'position');  
  set(hax2,'position',[pos(1)+pos(3)+0.05 pos(2) pos(3)/4 pos(4)]);
  xlabel('$\vphantom{\Sigma}$','interpreter','latex');
  
  % plot
  Rdom_ = linspace(Rdom(1),Rdom(end),60);
  plot(Pr1_*mom1_(2,1)/mom1(1),Rdom,'b', ...
       normpdf(Rdom_,mom1_(2,2),mom1_(2,3)),Rdom_,'r--');
  yline(mom1_(2,2),'r--');
  axis tight; v = axis;
  v([1 3:4]) = [0 Rdom(1) Rdom(end)]; axis(v);
  set(hax2,'ytick',1.3:0.1:1.7,'yticklabel',[]);
  set(hax2,'xtick',[]);
% $$$   print -depsc ~/Desktop/SIS_filtered_conditionalR0.eps
end

% $$$   % Jacobian of map (Sigma,R0,gamma) --> (alpha,beta,gamma)_OU
% $$$   syms Sigma_ R0_ gamma_ h_ % (note: h is fixed)
% $$$   Map = [Sigma_*(1-1/R0_); ...
% $$$          exp(-gamma_*h_*(R0_-1)); ...
% $$$          (R0_/Sigma_)/(1-exp(-2*gamma_*h_*(R0_-1)))];
% $$$   d1 = diff(Map,Sigma_);
% $$$   d2 = diff(Map,R0_);
% $$$   d3 = diff(Map,gamma_);
% $$$   Jac = [d1 d2 d3];
% $$$   % nonsingular at operating point
% $$$   J0 = double(subs(Jac,struct('R0_',R0,'gamma_',gamma,'Sigma_',Sigma,'h_',h)));
% $$$   eig(J0)