%FULLSTATE_BAYESIAN_KALMAN_SIS Bayesian inference, SIS-model.
%   This code investigates the Kalman-posterior for a SIS
%   point-process.

% S. Engblom 2021-08-17 (R0 < 1)
% S. Engblom 2021-07-30

%% 1) generate data

if ~exist('illustrate','var')
  illustrate = true;
end

% setup SIS-model (notation from the main paper)
Sigma = 1000;
if ~exist('R0','var')
  R0 = 1.5; % [1.25,4] seems ok
end
if ~exist('gamma','var')
  % days^-1 to recovery
  gamma = 1; % [1/4,1] seems ok
end
beta = R0*gamma;

Tend = 100; % note: should be considered relative to gamma
N = Tend+1;
tsample = linspace(0,Tend,N);
h = tsample(2)-tsample(1);
if R0 > 1
  X0 = [NaN; ceil(1e-2*Sigma)]; % [S I]
else
  X0 = [NaN; ceil(0.5*Sigma)];
end

%X0 = [NaN; ceil(Sigma*(1-1/R0))]; % (start at stationary mean)
X0(1) = Sigma-X0(2);

if ~exist('umod','var')
  umod.vol = Sigma;
  umod.D = sparse(zeros(2));
  umod = rparse(umod,{'S+I > Beta*S*I/vol > I+I', ...
                      'I > Gamma*I > S' ...
                      'S > (I == 0) > I'}, ... % (prevents extinction)
                {'S' 'I'},{'Beta' 'gdata' 'Gamma' 'gdata'},'SIS.c');
  % note: a better model could be to rather add
  %    S > Eta*S > I
  % which yields a full 3-parameter model easily done in 1D
  umod.sd = 1;
  umod.u0 = X0;
  umod.tspan = tsample;
end
umod.gdata = [beta gamma];

% simulate
if ~exist('compile','var')
  compile = true;
end
umod = urdme(umod,'solver','ssa','seed',1,'compile',compile);
Xctmc = umod.U;

if illustrate
  figure(1), clf,
  plot(tsample,Xctmc(2,:),'r'); hold on
end

% log-likelihood
logL = @SIS_kalman;

% parameter transformations
if R0 <= 1
  % linearize at I = ?
  error('Must decide were to linearize!');
  mu = gamma*(1-R0);
  k = 0;
  sigma = 1;
end

% OU-native
mu = gamma*(R0-1);
k = gamma*Sigma*(R0-1)^2/R0;
sigma = sqrt(2*gamma*Sigma*(R0-1)/R0);

% OU-alternative
atrue = k/mu;
btrue = exp(-mu*h);
gtrue = 2*mu/(sigma^2*(1-exp(-2*mu*h)));

if illustrate
  X_OU = OU_process(k,mu,sigma,tsample,X0(2));
  plot(tsample,X_OU,'b--');
end

% compute the log-likelihood over a mesh
%gdom = linspace(gamma*0.5,gamma*1.5,25);
%Rdom = linspace(R0*0.9,R0*1.1,23);
% finer resolution:
gdom = linspace(gamma*0.5,gamma*1.5,91);
Rdom = linspace(R0*0.9,R0*1.1,51);
L = zeros(numel(gdom),numel(Rdom),numel(tsample));

hw = waitbar(0,'Computing...');
for i = 1:numel(gdom)
  waitbar(i/numel(gdom),hw);
  for j = 1:numel(Rdom)
    g_ = gdom(i);
    b_ = Rdom(j)*g_;
    [L_,xx,PP] = logL([b_ g_ Sigma h],Xctmc(2,:), ...
                      X0(2),1e-6,4);
    L(i,j,:) = reshape(L_,1,1,[]);
  end
end
close(hw);

% investigation of filter output
% $$$ [L_,xx,PP] = logL([beta gamma Sigma h],Xctmc, ...
% $$$                   X0,1e-6*eye(2));
% $$$ plot(tsample,xx(2,:,1),'b:');
% $$$ err = sqrt(permute(PP(1,1,:,1),[1 3 2 4]));
% $$$ patch([tsample fliplr(tsample)], ...
% $$$       [xx(2,:,1)+err fliplr(xx(2,:,1)-err)], ...
% $$$       [0 0 1],'FaceAlpha',0.15, ...
% $$$       'LineStyle','none','FaceColor',[0 0 1], ...
% $$$       'HandleVisibility','off');
% $$$ plot(tsample,xx(2,:,2),'b--.');

% remove 1st L for consistency with other experiments (X0 considered
% known)
L = L(:,:,2:end);
L = cumsum(L,3);
Pr = L(:,:,end);

% visualize the likelihood/posterior
if illustrate
  figure(2), clf,
  contour(Rdom,gdom,Pr,40);
  xlabel('R_0');
  ylabel('\gamma');
end

% numerical posterior
Pr = exp(Pr-max(Pr(:)));
hg = gdom(2)-gdom(1);
hR = Rdom(2)-Rdom(1);
Pr = Pr/sum(Pr(:))/(hg*hR);
