%KALMAN_SIS_MARGINAL Posterior marginal densities for SIS-model.
%   Relies heavily on FULLSTATE_BAYESIAN_KALMAN_SIS.

% S. Engblom 2021-08-08

clear all

% (note: R0 and gamma cannot have different range lengths, range or
% scalar triggers different workings of the code)
experiment = 1; % 1 or 2
if experiment == 1
  R0range = linspace(1.1,3.5,16);
  gammarange = 1;
else
  R0range = 1.5;
  gammarange = linspace(0.1,1,16);
end
nrange = max(numel(R0range),numel(gammarange));

% [truth; theory sigma; ML-estimate; estimated sigma]
R0res = zeros(4,nrange);
R0res(1,:) = R0range;
gammares = zeros(4,nrange);
gammares(1,:) = gammarange;

compile = true;
illustrate = nrange == 1;
for ip = 1:nrange
  R0 = R0res(1,ip);
  gamma = gammares(1,ip);

  fullstate_bayesian_kalman_SIS
  compile = false;

  % marginal density R0
  Pr1_ = sum(Pr,1)*hg; % "reasonable" scaling 

  % immediate numerical attempt
  mom1 = [trapz(Rdom,Pr1_) trapz(Rdom,Pr1_.*Rdom) trapz(Rdom,Pr1_.*Rdom.^2)];
  mom1(3) = sqrt(mom1(3)-mom1(2)^2);

  % normal fit and deduced moments
  mom1_ = [mom1; normfit_nomass(Rdom,Pr1_,mom1)];

  % theoretical values
  std1est = [sqrt(2/(gamma*h*(R0-1))*R0^3/(Sigma*N)) ...
             sqrt((1+btrue)/(1-btrue)*R0^3/(Sigma*N))];
  R0res(2:4,ip) = [std1est(1); mom1_(2,2); mom1_(2,3)];

  % visualization
  if nrange == 1
    mom1_
    std1est
    mom1_(:,3)./std1est(end)
    figure(3), clf,
    plot(Rdom,Pr1_*mom1_(2,1)/mom1(1),'b', ...
         Rdom,normpdf(Rdom,mom1_(2,2),mom1_(2,3)),'r');
    xline(mom1_(2,2),'r--');
    xlabel('R_0');
    v = axis; v(3) = 0; axis(v);
  end

  % marginal density gamma
  Pr2_ = sum(Pr,2)*hR;
  mom2 = [trapz(gdom,Pr2_') trapz(gdom,Pr2_'.*gdom) trapz(gdom,Pr2_'.*gdom.^2)];
  mom2(3) = sqrt(mom2(3)-mom2(2)^2);
  mom2_ = [mom2; normfit_nomass(gdom,Pr2_',mom2)];
  std2est = [sqrt((1+btrue)/(1-btrue)*R0^3/(R0-1)^2*gamma^2/(Sigma*N)) ...
             sqrt((1+btrue)*(1-btrue)/N*1/(btrue^2*h^2*(R0-1)^2))];
  std2est(2) = sqrt(sum(std2est.^2));
  % puzzling that std2est(2) = sqrt(prod(std2est)) works well, but why?
  % also, mom2_(2,3) is sometimes clearly too optimistic, like if the
  % posterior is "too narow"? seems the UQ derived from Bayesian-OU is
  % inaccurate for gamma which governs the transient where indeed the
  % OU is a bit too fast - perhaps btrue = exp(-gamma*h*(R0-1)) should
  % be replaced by exp(-gamma_eff*h*(R0-1)), where gamma_eff is some
  % kind of effective value?
  gammares(2:4,ip) = [std2est(2); mom2_(2,2); mom2_(2,3)];
  if nrange == 1
    mom2_
    std2est
    mom2_(:,3)./std2est(end)      
    figure(4), clf,
    plot(gdom,Pr2_*mom2_(2,1)/mom2(1),'b', ...
         gdom,normpdf(gdom,mom2_(2,2),mom2_(2,3)),'r');
    xline(mom2_(2,2),'r--');
    xlabel('\gamma');
    v = axis; v(3) = 0; axis(v);
  end
end

% no range plots
if nrange == 1, return; end

% (1) plots for R0range
if numel(R0range) > 1
  R0range_ = [R0range fliplr(R0range)];
  gammarange = gammares(1,:);
  gammarange_ = [gammarange fliplr(gammarange)];

  figure(5), clf,
  yline(1,'b'); hold on
  patch(R0range_, ...
        [R0res(3,:)+2*R0res(2,:) fliplr(R0res(3,:)-2*R0res(2,:))]./R0range_, ...
        [0 0 1],'FaceAlpha',0.15, ...
        'LineStyle','none','FaceColor',[0 0 1], ...
        'HandleVisibility','off');
  plot(R0range,(R0res(3,:)+2*R0res(2,:))./R0range,'b:', ... 
       R0range,(R0res(3,:)-2*R0res(2,:))./R0range,'b:');
  patch(R0range_, ...
        [R0res(3,:)+2*R0res(4,:) fliplr(R0res(3,:)-2*R0res(4,:))]./R0range_, ...
        [1 0 0],'FaceAlpha',0.15, ...
        'LineStyle','none','FaceColor',[1 0 0], ...
        'HandleVisibility','off');
  plot(R0range,R0res(3,:)./R0range,'rx');
  axis tight
  xlabel('$R_0$','interpreter','latex');
  ylabel('$\hat{R_0}/R_0$','interpreter','latex');
  set(gca,'xtick',1:0.5:3,'xticklabel',{'' '' 2 '' 3}, ...
          'ytick',0.95:0.025:1.025,'yticklabel',{0.95 '' 1 ''});
  
  set(gcf,'PaperPositionMode','auto');
  set(gcf,'Position',[100 100 350 150]);
  %print -dpdf ~/Desktop/SIS_marginalR0.pdf % *** must be manually cropped
  
  figure(6), clf,
  yline(1,'b'); hold on
  patch(R0range_, ...
        [gammares(3,:)+2*gammares(2,:) fliplr(gammares(3,:)-2*gammares(2,:))]./gammarange_, ...
        [0 0 1],'FaceAlpha',0.15, ...
        'LineStyle','none','FaceColor',[0 0 1], ...
        'HandleVisibility','off');
  plot(R0range,(gammares(3,:)+2*gammares(2,:))./gammarange,'b:', ... 
       R0range,(gammares(3,:)-2*gammares(2,:))./gammarange,'b:');
  plot(R0range,gammares(3,:)./gammarange,'rx');
  patch(R0range_, ...
        [gammares(3,:)+2*gammares(4,:) fliplr(gammares(3,:)-2*gammares(4,:))]./gammarange_, ...
        [1 0 0],'FaceAlpha',0.15, ...
        'LineStyle','none','FaceColor',[1 0 0], ...
        'HandleVisibility','off');
  axis tight
  set(gca,'xtick',1:0.5:3,'xticklabel',{'' '' 2 '' 3}, ...
          'ytick',0.5:0.5:2);
  xlabel('$R_0$','interpreter','latex');
  ylabel('$\hat{\gamma}/\gamma$','interpreter','latex');
end

% (2) plots for gammarange
if numel(gammarange) > 1
  R0range = R0res(1,:);
  R0range_ = [R0range fliplr(R0range)];
  gammarange_ = [gammarange fliplr(gammarange)];    

  figure(7), clf,
  yline(1,'b'); hold on
  patch(gammarange_, ...
        [R0res(3,:)+2*R0res(2,:) fliplr(R0res(3,:)-2*R0res(2,:))]./R0range_, ...
        [0 0 1],'FaceAlpha',0.15, ...
        'LineStyle','none','FaceColor',[0 0 1], ...
        'HandleVisibility','off');
  plot(gammarange,(R0res(3,:)+2*R0res(2,:))./R0range,'b:', ... 
       gammarange,(R0res(3,:)-2*R0res(2,:))./R0range,'b:');
  patch(gammarange_, ...
        [R0res(3,:)+2*R0res(4,:) fliplr(R0res(3,:)-2*R0res(4,:))]./R0range_, ...
        [1 0 0],'FaceAlpha',0.15, ...
        'LineStyle','none','FaceColor',[1 0 0], ...
        'HandleVisibility','off');
  plot(gammarange,R0res(3,:)./R0range,'rx');
  axis tight
  xlabel('$\gamma$','interpreter','latex');
  ylabel('$\hat{R_0}/R_0$','interpreter','latex');
  set(gca,'xtick',0.1:0.1:1,'xticklabel',{'' '' '' '' 0.5 '' '' '' '' 1}, ...
          'ytick',0.95:0.025:1.05,'yticklabel',{0.95 '' 1 '' 1.05});
  set(gcf,'PaperPositionMode','auto');
  set(gcf,'Position',[100 100 350 150]);
  %print -dpdf ~/Desktop/SIS_marginalR0_gammarange.pdf

  figure(8), clf,
  yline(1,'b'); hold on
  patch(gammarange_, ...
        [gammares(3,:)+2*gammares(2,:) fliplr(gammares(3,:)-2*gammares(2,:))]./gammarange_, ...
        [0 0 1],'FaceAlpha',0.15, ...
        'LineStyle','none','FaceColor',[0 0 1], ...
        'HandleVisibility','off');
  plot(gammarange,(gammares(3,:)+2*gammares(2,:))./gammarange,'b:', ... 
       gammarange,(gammares(3,:)-2*gammares(2,:))./gammarange,'b:');
  plot(gammarange,gammares(3,:)./gammarange,'rx');
  patch(gammarange_, ...
        [gammares(3,:)+2*gammares(4,:) fliplr(gammares(3,:)-2*gammares(4,:))]./gammarange_, ...
        [1 0 0],'FaceAlpha',0.15, ...
        'LineStyle','none','FaceColor',[1 0 0], ...
        'HandleVisibility','off');
  axis tight
  set(gca,'xtick',0.1:0.1:1,'xticklabel',{'' '' '' '' 0.5 '' '' '' '' 1}, ...
          'ytick',0:0.5:2);
  xlabel('$\gamma$','interpreter','latex');
  ylabel('$\hat{\gamma}/\gamma$','interpreter','latex');
end

return;

% derivation of uncertainty estimates:
%
% if y = f(x), then Cov_y ~ grad f(x0)*Cov_x*[grad f(x0)]' for x0 = mean(x)
syms Sigma h N R0 gamma alpha0 beta0 gamma0
Map = [1/(1-Sigma\alpha0); -log(beta0)/h*(Sigma-alpha0)/alpha0];
d1 = diff(Map,alpha0);
d1 = subs(d1,[alpha0 log(beta0)],[Sigma*(1-1/R0) -gamma*h*(R0-1)]);
d1 = collect(collect(d1,R0),[Sigma h gamma]);
d2 = diff(Map,beta0);
d2 = subs(d2,alpha0,Sigma*(1-1/R0));
d2 = collect(d2,[beta0 h Sigma]);
d3 = diff(Map,gamma0); % = 0
Jac = [d1 d2 d3];
Q = diag([1/(1-beta0)^2*1/gamma0 (1-beta0^2) 2*gamma0^2]/N);
Q = subs(Q,gamma0,R0/Sigma*1/(1-beta0^2));

UQ = Jac*Q*Jac.';
UQ = simplify(UQ);
UQ(1,1) % var(R0)
UQ(2,2) % var(gamma) - more difficult to simplify automatically
