function x = OU_process(k,mu,sigma,tspan,X0)
%OU_PROCESS Ornstein-Uhlenbeck process simulation.
%   X = OU_PROCESS(k,mu,sigma,tspan,X0) samples an Ornstein-Uhlenbeck
%   process X at times tspan with initial data X(tspan(1)) = X0. The
%   process is defined as the solution to the SDE
%     dX(t) = k-mu X dt + sigma dW(t)
%   with W(t) a Wiener process.
%
%   Example:
%     tspan = linspace(0,10);
%     k = 1; mu = 2; sigma = 0.5;
%     X = OU_process(k,mu,sigma,tspan,0);
%     Xmean = k/mu*(1-exp(-mu*tspan));
%     Xstd = sigma/sqrt(2*mu)*sqrt(1-exp(-2*mu*tspan));
%     figure, plot(tspan,X,'b',tspan,Xmean,'r', ...
%       tspan,Xmean+Xstd,'k--',tspan,Xmean-Xstd,'k--');
%
%   Reference:
%     [1] S. Bronstein, S. Engblom, R. Eriksson: "Bayesian inference
%     in Epidemics: consistency and convergence", manuscript (2021).
%
%   See also RNG, RANDN.

% S. Engblom 21-02-01 (Revision, sub-resolution warning added)
% S. Engblom 2019-05-20

% input check
narginchk(5,5);
if ~isscalar(k) || ~isscalar(mu) || ~isscalar(sigma) || ~isscalar(X0) ...
      || numel(tspan) < 2
  error('Cannot parse input arguments.');
end

% (1) compute corrected numerical parameters per each time step such
% that a forward Euler step yields an exact step of the wanted
% OU-process
dt = diff(tspan);
mu_ = (1-exp(-mu*dt))./dt;
k_ = mu_*(k/mu);
sigma_ = sigma*sqrt(mu_.*(2-mu_.*dt)/(2*mu));
% (solved from Prop. 2.1 in [1])
if any(dt >= 1./mu_)
  warning(sprintf(['OU-process not resolved. Increase the sampling ' ...
                   'rate such that delta t < %f.'],min(1./mu_)));
end

% (2) then do forward Euler using these numerical parameters
xx = X0;
x = [X0 zeros(1,numel(dt))];
for i = 1:numel(dt)
  xx = xx+(k_(i)-mu_(i)*xx)*dt(i)+sigma_(i)*sqrt(dt(i))*randn;
  x(i+1) = xx;
end
