function [L,x,P] = SIS_kalman(p,y,x0,P0,NSTEPS)
%SIS_KALMAN Kalman filter for SIS-process.
%   [L,x,P] = SIS_KALMAN(p,y,x0,P0,NSTEPS) uses the Kalman filter
%   defined by p and the measurements defined in y, to produce a
%   likelihood L, state estimates x, and covariance matrices P. x0 and
%   P0 set the initial state and covariance, respectively. NSTEPS is
%   the (integer) number of steps taken per measurement with default =
%   1.
%
%   L is a length size(y,2) vector with L(m) the marginal likelihood
%   contribution of y(:,m).
%
%   The state estimate x has the following structure: x(:,m,1) and
%   x(:,m,2) contain the a priori- and a posteriori state estimates at
%   index m, from the measured data y(:,1:m-1) and y(:,1:m),
%   respectively. P(:,:,m,1) and P(:,:,m,2) contain the a priori- and
%   a posteriori covariances correspondingly.
%
%   The SIS-process is described by p = [beta gamma Sigma h].

% S. Engblom 2021-07-30

% parameters
if nargin < 5, NSTEPS = 1; end
beta = p(1);
gamma = p(2);
Sigma = p(3);
h = p(4);

% allocate filter output
ntime = size(y,2);
x = zeros(size(x0,1),ntime,2);
P = zeros(size(x0,1),size(x0,1),ntime,2);
L = zeros(1,size(y,2));
d2pi = size(y,1)*log(2*pi);

% initial state
xx = x0;
PP = P0;

% Kalman notation
h_ = h/NSTEPS; % (h_ the Kalman step, h the measurement step)
I = eye(1);
F1 = beta/Sigma*h_;
F2 = -gamma*h_;

Q1 = beta/Sigma*h_;
Q2 = gamma*h_;
H = I;
R = 1e-6*I;

for m = 1:ntime
  % prior on states *after* the initial state
  if m > 1
    % a priori
    for i = 1:NSTEPS
      F = I+F1*(Sigma-xx)+F2;
      Q = Q1*(Sigma-xx)*xx+Q2*xx;
      xx = F*xx;
      PP = F*PP*F'+Q;
    end
  end

  % a priori
  x(:,m,1) = xx;
  P(:,:,m,1) = PP;

  % pre-fit residual
  yy = y(:,m)-H*xx;

  % update
  S = H*PP*H'+R;
  K = PP*H'/S;

  % updated likelihood
  L(m) = -0.5*(yy'*(S\yy)+log(det(S))+d2pi);

  % a posteriori
  xx = xx+K*yy;
  PP = (I-K*H)*PP;
  x(:,m,2) = xx;
  P(:,:,m,2) = PP;
end
