function [L,xx,W] = SIS_kalman_filtered(p,y,c,x0,P0,NSTEPS,M)
%SIS_KALMAN_FILTERED Kalman filter for SIS-process.
%   [L,xx,W] = SIS_KALMAN_FILTERED(p,y,c,x0,P0,NSTEPS,M) uses the
%   (particle-) Kalman filter defined by p and the measurements
%   defined by (y,c), with c the cutoff, to produce a likelihood L,
%   and state estimates (xx,W). Inputs x0 and P0 set the initial state
%   and covariance, respectively, NSTEPS is the number of Kalman steps
%   per data measurement, and M is the number of particles used (M =
%   10 is the default).
%
%   The data is understood as y(t) = x(t) >= c, where x(t) is the full
%   state and c is the cutoff.
%
%   L is a length size(y,2) vector with L(m) the marginal likelihood
%   contribution of y(:,m).
%
%   WW is numel(xx)-by-numel(L) and contains the weights for the state
%   mesh xx, normalized such that sum(WW) is one. Latent state
%   estimates x(t) can thus be compute via x(t) = xx*W.
%
%   The SIS-process is described by p = [beta gamma Sigma h].

% S. Engblom 2021-08-12 (derived from sis_kalman())
% S. Engblom 2021-07-30

% inputs
if nargin < 7
  M = 10;
  if nargin < 6
    NSTEPS = 1;
  end
end

% parameters
beta = p(1);
gamma = p(2);
Sigma = p(3);
h = p(4);

% OU-versions (convenient)
R0 = beta/gamma;
if R0 <= 1, error('Code does not allow R0 <= 1.'); end
atrue = Sigma*(1-1/R0);
btrue = exp(-gamma*(R0-1)*h);
gtrue = Sigma\R0*1/(1-btrue^2);

% stationary std
sigma_stat = sqrt(1/gtrue/(1-btrue^2));

% define uniform mesh (with _some_ adaptivity):
xx = [0.9*c norminv([1/(M+1) 1-1/(M+1)],atrue,sigma_stat) 1.1*c];
xx(xx < 0) = 0;
xx(xx > Sigma) = Sigma;
xx = linspace(min(xx),max(xx),M);
% with probability mass m = 1/M, this is the idea:
%   [m xx(1) m xx(2) ... m xx(M) m]
% or
%   [m int_bnds(2) m int_bnds(3) ... m int_bnds(M) m]

% variance around each point xx
PP = 1e-6*ones(size(xx));

% initial weights: stationary assumption
int_bnds = [-inf 0.5*(xx(1:end-1)+xx(2:end)) inf];
ww_stat = diff(normcdf(int_bnds,atrue,sigma_stat));

% interpolation of probability mass according to cutoff c
ic = find(int_bnds >= c,1,'first');
cfac = (c-int_bnds(ic-1))/(int_bnds(ic)-int_bnds(ic-1));
ic = ic-1; % points into xx

% precomputed table of Dirac mesures at xx forwarded in time according
% to the Kalman filter
h_ = h/NSTEPS; % (h_ the Kalman step, h the measurement step)
I = eye(1);
F1 = beta/Sigma*h_;
F2 = -gamma*h_;
Q1 = beta/Sigma*h_;
Q2 = gamma*h_;

TABLE = zeros(M);
for j = 1:M
  mx = xx(j);
  sigma_ = PP(j);
  for i = 1:NSTEPS
    F = I+F1*(Sigma-mx)+F2;
    Q = Q1*(Sigma-mx)*mx+Q2*mx;
    mx = F*mx;
    sigma_ = F*sigma_*F'+Q;
  end
  TABLE(:,j) = diff(normcdf(int_bnds,mx,sqrt(sigma_))');
end

% allocate filter output
ntime = size(y,2);
x = zeros(size(x0,1),ntime,2);
P = zeros(size(x0,1),size(x0,1),ntime,2);
L = zeros(1,size(y,2));

% stationary prior for first state
ww = ww_stat;
% form posterior:
if y(1) == 1
  w_ = ww(ic)*(1-cfac);
  ww(xx < c) = 0;
  ww(ic) = w_;
else
  w_ = ww(ic)*cfac;
  ww(xx >= c) = 0;
  ww(ic) = w_;
end
p_ = sum(ww);
L(1) = log(p_);
ww = ww/p_;
if nargout > 2
  W = zeros(numel(xx),numel(L));
  W(:,1) = ww(:);
end

% $$$ % initial state
% $$$ xx = x0;
% $$$ PP = P0;

% $$$ % "cheating": we use here that x0 = 0 exactly and considered known
% $$$ ix = find(int_bnds >= 0,1,'first');
% $$$ ww = zeros(size(ww_stat));
% $$$ ww(ix-1) = 1;
% $$$ L(1) = 0;
% $$$ if nargout > 2
% $$$   W = zeros(numel(xx),numel(L));
% $$$   W(:,1) = ww(:);
% $$$ end

for i = 2:numel(y)
  % add weight contributions from all evolved particles at the mesh
  % points xx, thus forming the prior density for state x(i)
  ww = ww*TABLE';
  % determine posterior density: x(i) | y(i)
  if y(i) == 1
    w_ = ww(ic)*(1-cfac);
    ww(xx < c) = 0;
    ww(ic) = w_;
  else
    w_ = ww(ic)*cfac;
    ww(xx >= c) = 0;
    ww(ic) = w_;
  end
  p_ = sum(ww);
  L(i) = log(p_);
  ww = ww/p_;
  if nargout > 2
    W(:,i) = ww(:);
  end
end
